/*
 Alina Campan - Attribute.java
 -----------------------------
 Date Created: 	08-10-2006
 Date Modified:	03-17-2008
 -----------------------------
  It contains the following class:
 	
	- Attribute - represents an attribute that characterizes the data for p-sensitive k-anonymity; 
*/


/** 
* An attribute can be of one of the attribute types defined in class AttributeType; 
* currently there are defined and used two such attribute types: numeric or categorical 
* attributes.
* 
* An attribute has a unique name between the set of all attributes characterizing the data.
* 
* @author Alina Campan 
*/
public class Attribute {
	String name;  	//Attribute name
	int type;		//Attribute type (categorial or numeric)
	Hierarchy h;	//Attribute hierarchy
	double w;		//Attribute weight (only for sensitive attributes)
	
  	/** 
	* Constructs an attribute
	* @param name	the attribute name.
	* @param type	is an integer that indicates the attribute type: AttributeType.NUMERIC for a numeric attribute, AttributeType.CATEGORICAL for a categorical attribute.
	*
	*/
	Attribute (String name, int type) {
		this.name = name;
		this.type = type;
		h = null;
		w = -1;
	}
	
	/**
	 * Constructs a sensitive attribute
	 * @param name the attribute name.
 	 * @param type	is an integer that indicates the attribute type: AttributeType.NUMERIC for a numeric attribute, AttributeType.CATEGORICAL for a categorical attribute.
	 * @param w is the weight of the attribute
	 */
	Attribute (String name, int type, double w){
		this(name,type);
		this.w = w;
	}

	
	/**
	 * Gets the weight of the current attribute.
	 * @return Returns the w.
	 */
	public double getW() {
		return w;
	}
	
	/**Sets the weight of the current attribute.
	 * @param w The w to set.
	 */
	public void setW(double w) {
		this.w = w;
	}

	/** 
	* Sets the name of the current attribute.
	* @param n	the name of the attribute.
	*/
	public void setName (String name) {
		this.name = name;
	}

	/** 
	* Gets the name of the current attribute.
	* @return	the name of the attribute.
	*/  
	public String getName() {
		return name;
	}
  
	/** 
	* Sets the type of the current attribute.
	* @param t	an int that indicates the attribute type (AttributeType.NUMERIC for a numeric attribute, AttributeType.CATEGORICAL for a categorical attribute).
	*/
	public void setType (int type) {
		this.type = type;
	}
  
	/** 
	* Gets the type of the current attribute, which is one of the symbolic constants defined in class AttributeType.
	* @return	the type of the attribute.
	*/  
	public int getType() {
		return type;
	}
	
	/** 
	* Sets the hierarchy of the current attribute; used only for categorial attributes, where it indicates a tree hierarchy 
	* of the attribute domain values.
	* @param hier	the hierarchy for the attribute.
	*/
	public void setHierarchy (Hierarchy h) {
		this.h = h;
	}
  
	/** 
	* Gets the hierarchy associated to the current attribute.
	* @return	the hierarchy associated to the attribute.
	*/  
	public Hierarchy getHierarchy() {
		return h;
	}
  
	/**
	* Writes to the standard output the attribute name and type.
	*/
	public void out() {
		System.out.println(name + ' ' + type);
 	}
}
