

/*
 Alina Campan - CategoricalHNode.java
 -----------------------------
 Date Created: 	08-10-2006
 Date Modified:	03-18-2008
 -----------------------------
 
 It contains the following class:
 	
 	- CategoricalHNode - represents a node in a categorical attribute hierarcy

*/


/** 
* CategoricalHNode is a descendant class of the abstract class HNode and represents a node in 
* a categorical attribute's hierarchy. 
*
* The node value is a string for CategoricalHNode.
*
* @author Alina Campan
*/
public class CategoricalHNode extends HNode {
	
	/** 
	* Constructs a categorical node with data value set to null, by calling the corresponding constructor 
	* of the parent class HNode.
	*/
	CategoricalHNode () {
		super();
	} 
	
	/** 
	* Constructs a categorical node, by calling the corresponding constructor of the parent class HNode.
	* @param d	the data value to be stored in the current node; the actual parameter will be a String (the data in a categorical node is of this type).
	*/
	CategoricalHNode (Comparable d) {
		super(d);
	} 

	/** 
	* Implements the corresponding abstract method of the parent class HNode and sets the data in the current node.
	* @param o	the data from the current node
	*/
	public void setData (String o) {
		data = o;
	}

	/** 
	* Implements the corresponding abstract method of the parent class HNode and searches the value s in the current node.
	* @param s	the value to be searched
	* @return	0 if the value is found, not 0 otherwise.
	*/
	public int searchData (Comparable s) {
		return ((String) data).compareTo((String) s);
	}

	/** 
	* Implements the corresponding abstract method of the parent class HNode and converts the current data to string.
	* @return	a String representing the current data.
	*/
	public String toString () {
		return ((String) data);
	}
}
