/*
 Alina Campan - HNode.java
 -----------------------------
 Date Created: 	08-10-2006
 Date Modified:	03-17-2008
 -----------------------------
 
 It contains the following class:
 	
 	- HNode - abstract class; represents a node in an attribute hierarchy.
*/


/** 
* HNode is an abstract class that represents a node in an attribute hierarchy.
*
* @author Alina Campan
*/
public abstract class HNode {
	Comparable data; 	// the value in that node
	HNode[] desc; 		// descendant nodes
	HNode parent; 		// parent node
	
	/** 
	* Constructs a node with data value set to null.
	*/
	HNode() {
		data = null;
		desc = null;
		parent = null;
	}

	/** 
	* Constructs a node.
	* @param d	the data value to be stored in the current node.
	*/
	HNode(Comparable d) {
		data = d;
		desc = null;
		parent = null;
	}
	
	/** 
	* Gets the data stored in a node.
	* @return	data stored in a node.
	*/
	public Comparable getData() {
		return data;
	}
	
	/** 
	* Sets the parent of the node.
	* @param p	the parent of the current node.
	*/
	public void setParent (HNode p) {
		parent = p;
	}
	
	/** 
	* Gets the parent of the node.
	* @return	the parent of the current node.
	*/
	public HNode getParent() {
		return parent;
	}
	
	/** 
	* Abstract method that sets the current data.
	* @param o	the data for the current node
	*/
	public abstract void setData (String o);
		
	/** 
	* Abstract method for searching the value s in the current node.
	* @param s	the value to be searched
	* @return	0 if the value is found, not 0 otherwise.
	*/
	public abstract int searchData (Comparable s);

	/** 
	* Abstract method for converting the current data to string.
	* @return	a String representing the current data.
	*/
	public abstract String toString();
}
