/*
 Alina Campan - Interval.java
 -----------------------------
 Date Created: 	08-10-2006
 Date Modified:	03-17-2008
 -----------------------------
 
 It contains the following class:
 	
 	- Interval - represents an interval of doubles
*/

/** 
* Interval represents an closed interval.
* The interval can be seen as [min, max], where min and max are two double values.
*
* @author Alina Campan
*/

import java.util.*;


public class Interval implements Comparable{
	double min, max;
	/** 
	* Constructs the interval: [-Double.MAX_VALUE, Double.MAX_VALUE]
	*/
	Interval () {
		min = Double.MAX_VALUE;
		max = - Double.MAX_VALUE;
	}
	
	/** 
	* Constructs an interval
	* @param s	is a string of the form "min-max"
	*			Ex: 10-24.
	*			NOTE: both min and max are assumed to be positive!
	*/
	Interval (String s) {
		StringTokenizer st = new StringTokenizer(s, "-");
		min = Double.parseDouble(st.nextToken());
		max = Double.parseDouble(st.nextToken());
	}
	
	/** 
	* Constructs an interval
	* @param vmin	left value of the interval
	* @param vmax	right value of the interval
	*/
	Interval (double vmin, double vmax) {
		min = vmin;
		max = vmax;
	}

	/** 
	* Constructs an interval with a single value
	* @param v		both left and right value of the interval
	*/
	Interval (double v) {
		min = v;
		max = v;
	}
	
	/** 
	* Compares this interval to another Object.
	* @param s		the object to be compared
	* @return 		the value 0 if the argument is an interval equal to this interval (both intervals have the same min and max)
	*               the value 1 otherwise.
	*/
	public int compareTo (Object s) {
		if ((min == ((Interval) s).min) && (max == ((Interval) s).max)) return 0;
		else return 1;
	}
	
	/** 
	* Returns the string representation of the interval.
	* @return 		the string representation of the interval in the format "min-max"
	*/
	public String toString () {
		return new String (min + "-" + max);
	}
	
	/** 
	* Returns the left value of the interval.
	* @return 		the left value from the interval.
	*/
	public double getMin() {
		return min;
	}

	/** 
	* Returns the right value of the interval.
	* @return 		the right value from the interval.
	*/
	public double getMax() {
		return max;
	}
	
	/** 
	* Sets the left value of the interval.
	* @param min 		the left value from the interval.
	*/
	public void setMin(double min) {
		this.min = min;
	}

	/** 
	* Sets the right value of the interval.
	* @param max 		the right value from the interval.
	*/
	public void setMax(double max) {
		this.max = max;
	}
	
	/** 
	* Checks if this interval intersects the parameter interval.
	* @param i 		an interval.
	* @return 		0 if this and i intersects. 
	*				1 othervise.
	*/
	public int intersect(Interval i) {
		if ((getMax()<i.getMin()) || (i.getMax()<getMin())) return 1;
		else return 0;
	}
	
	/** 
	* Checks if the double value given as parameter belongs to this interval.
	* @param v 		a double value.
	* @return 		0 if v belongs to this interval. 
	*				1 othervise.
	*/
	public int belong(double v) {
		if ( (getMin()<=v) && (getMax()>=v) ) return 0;
		else return 1;
	}

}
