/*
 Alina Campan - NumericHNode.java
 -----------------------------
 Date Created: 	08-10-2006
 Date Modified:	03-17-2008
 -----------------------------
 
 It contains the following class:
 	
 	- NumericHNode - represents a node in a numerical attribute hierarcy
*/


/** 
* NumericHNode is a descendant class of the abstract class HNode and represents a node in 
* a numerical attribute's hierarchy. 
*
* The node value is an Interval for NumericHNode.
*
* @author Alina Campan
*/
public class NumericHNode extends HNode {

	/** 
	* Constructs a numeric node with data value set to null, by calling the corresponding constructor 
	* of the parent class HNode.
	*/
	NumericHNode () {
		super();
	} 

	/** 
	* Constructs a numeric node, by calling the corresponding constructor of the parent class HNode.
	* @param d	the data value to be stored in the current node; the actual parameter will be an Interval (the data in a numeric node is of this type).
	*/
	NumericHNode (Comparable d) {
		super(d);
	} 
	
	/** 
	* Implements the corresponding abstract method of the parent class HNode and sets the data in the current node.
	* @param o	the data from the current node; must be a string having the form "min-max", from which an Interval can be constructed.
	*/
	public void setData (String o) {
		Interval ad = new Interval(o);
		data = (Comparable) ad;
	}
	
	
	/** 
	* Implements the corresponding abstract method of the parent class HNode and searches the value s in the current node.
	* @param s	the value to be searched; must be a string having the form "min-max".
	* @return	0 if the value is found, not 0 otherwise.
	*/
	public int searchData (Comparable s) {
		return ((Interval) data).compareTo(new Interval((String) s));
	}
	
	/** 
	* Implements the corresponding abstract method of the parent class HNode and converts the current data to string.
	* @return	a String representing the current data, which is an Interval.
	*/
	public String toString () {
		return ((Interval) data).toString();
	}
}
