/*
 Alina Campan - P_sensitivity.java
 -----------------------------
 Date Created: 	05-20-2008
 Date Modified:	05-20-2008
 -----------------------------
 
 It contains the following class:
 	
	- SNGreedyAnonymization - implements (not yet! p-sensitivity) k-anonymization by greedy clustering algorithm for social networks.
*/


import java.util.*;
import java.io.*;
import java.sql.*;

/**
* Implements the p-sensitivity k-anonymization by greedy clustering algorithm for social networks.
*
*
*/
public class SNGreedyAnonymization {
	
	static String descriptor_file = "";
	static String dbname = "";
	static String ent_table = "";
	static String edges_table = "";
	static int k = 1;
	static int p = 1;
	static Descriptor d; 			// entity set descriptor
	static Cluster[] cl = null; 	// the equivalence classes
	static String clust_table;
	static String anon_ent_table;
	static String anon_edges_table;
	static String taskID;
	static double alpha, beta; 
	// alpha has to be 1 to almost emulate Zheleva; 
	// 0.5 to equally treat generalization IL and structural IL
	// 1 to only consider structural IL

	public static void main(String args[]) throws Exception {
		descriptor_file = args[0];
		dbname = args[1];
		ent_table = args[2];
		edges_table = args[3];
		k = Integer.parseInt(args[4]);
		p = Integer.parseInt(args[5]);
		clust_table = args[6];
		anon_ent_table = args[7];
		anon_edges_table = args[8];
		taskID = args[9];
		alpha = Double.parseDouble(args[10]);
		beta = 1 - alpha;
		
		Entity e;
		
	   	BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		
  	   	System.out.println("Descriptorul : ");
  	   	d = new Descriptor(descriptor_file);
  	   	d.out(); 
	    
	   	// reading the hierarchies for every categorical attribute
	   	Hierarchy[] h = new Hierarchy[d.getNoCategoricalQuaAttr()];
	   	int j=0;
	   	for (int i=0; i<d.getNoQuaAttr(); i++) {
			Attribute a = d.getQuaAttribute(i);
			if (a.getType()==AttributeType.CATEGORICAL){
				// System.out.println("Attribute " + a.getName() + " is categorical");
				h[j] = new Hierarchy("hierarchy_" + a.getName() + ".txt", a);
				a.setHierarchy(h[j]);
				j++;
				System.out.println("Finished creating hierarchy for attribute " + a.getName());
			}
	    }
	    
	    Connection con = DBConnect(dbname);
	    Statement stmt = con.createStatement();
	    ResultSet rs;

	    // create the csAll cluster summary
	    ClusterSummary csAll = new ClusterSummary(d);
	    createClusterTable(con, clust_table);
   		createAnonEntTable(con, anon_ent_table);
   		createAnonEdgesTable(con, anon_edges_table);
		

		// approximates csAll with: the root for every categorical attribute, and the correct [min-max] for every numerical attribute
	 	for (int i=0; i<d.getNoQuaAttr(); i++) {
	 		Attribute a = d.getQuaAttribute(i);
	 		switch (a.getType()) {
	 			case AttributeType.NUMERIC : 
	 				stmt.execute("select min(" + a.getName() + "), max(" + a.getName() + ") from " + ent_table);
	 				rs = stmt.getResultSet();
	 				rs.next();
					csAll.summ[i] = new Interval(rs.getDouble(1), rs.getDouble(2));
	 				break;
	 			case AttributeType.CATEGORICAL : 
	 				csAll.summ[i] = (String) a.getHierarchy().getRoot().getData();
	 				break;
	 		}
	 	}		
		System.out.println("csAll cluster summary is :");
		csAll.out();

		// all entitites are not yet placed in any cluster <=> make placed=false for all!
		stmt = con.createStatement();
		stmt.execute("update " + ent_table + " set placed=false");

		stmt.execute("select count(*) from " + ent_table);
		rs = stmt.getResultSet();
		rs.next();
		int noElements = rs.getInt(1);
		cl = new Cluster[noElements / k +1];
		int noCurCl = 0;

		while (true) {
		// execute repeatedly, until no unallocated entities remain - begin
				// select the seed to start a new cluster
				int seed = selectSeed(con, ent_table, noElements, noCurCl);
				if (seed == -1)
					break; // no more unselected elements to start and form a new cluster
		
				markSelectedNode(con, ent_table, seed);
				System.out.println("Selected seed unselected entity number " + seed);
		
				// create a new cluster with one element, the seed.
				Cluster newCl = new Cluster(d);
				e = new Entity (d, con, ent_table, seed);
				newCl.add(seed, e);
				addEntityToClusterTable(con, clust_table, noCurCl+1, seed);
		
				while (newCl.size() < k) {
					int bestNode = -1; // the best node to add next to the current cluster

					String s = "select ID from " + ent_table + " where placed=false;";
					stmt.execute(s);
					rs = stmt.getResultSet();
					
					double bestCombinedIL = Double.MAX_VALUE;
					double curCombinedIL;
					bestNode = -1;
						
					while (rs.next()) {
						int curEntID = rs.getInt(1);
						// System.out.println("Checking entity " + curEntID);
						e = new Entity (d, con, ent_table, curEntID);
						ClusterSummary pos_cs = newCl.getSummary().updateSummaryWithNewEntity(e);
						curCombinedIL = alpha * pos_cs.InformationLoss(csAll);
						curCombinedIL += beta * structuralDistance(newCl, curEntID, con, edges_table, noElements); // method call to compute structural distance between the current cluster and the candidate entity;
						// System.out.println("Entity's " + curEntID + " combined IL is " + curCombinedIL);
							
						if (curCombinedIL < bestCombinedIL) {
							bestCombinedIL = curCombinedIL;
							bestNode = curEntID;
						}
					}
					rs.close();
					
					if (bestNode == -1)
						break;
						
					e = new Entity (d, con, ent_table, bestNode);
					newCl.add(bestNode, e);
					markSelectedNode(con, ent_table, bestNode);
					addEntityToClusterTable(con, clust_table, noCurCl+1, bestNode);
					System.out.println("Entity " + bestNode + " added to cluster " + (noCurCl+1));
				}
				
				if (newCl.size() < k) {
					// disperse the cluster - NOT IMPLEMENTED YET!???
					break; // clustering ends
				}
				else {
					System.out.println("new cluster: " + newCl.toString());
					// System.out.println("new cluster summary is :");
					// newCl.getSummary().out();
					cl[noCurCl++] = newCl;
					// write anonymized cluster into anon_ent_table
					addClusterToAnonEntTable(con, edges_table, anon_ent_table, noCurCl, newCl);
				}
		// execute repeatedly, until no unallocated entities remain - end
		}

		// write anonymized edges into anon_edges_table
		populateAnonEdgesTable(con, edges_table, anon_edges_table);

		double gil = quantifyGeneralizationInfoLoss(csAll);
		double sil = quantifyStructureInfoLoss(con, anon_ent_table, anon_edges_table, noElements);
		System.out.println("InfoLoss caused by generalization is: " + gil);
		System.out.println("InfoLoss caused by structure modification is: " + sil);

 		String s = "insert into AnonymizationTask (TaskID, DbName, DbEntTable, DbEdgesTable, DescriptorFileName, K, P, AnonymizationAlg, DbClustTable, DbAnonEntTable, DbAnonEdgesTable) values ('T_" + 
		 					  taskID + "', '" + dbname + "', '" +
		 					  ent_table  + "', '" + edges_table  + "', '" +
		 					  descriptor_file + "', " + k + ", " +
		 					  p + ", '" + "SNK" + "', '" +
		 					  clust_table + "', '" + anon_ent_table + "', '" + anon_edges_table + "')";
 		stmt.execute(s);

		stmt.execute("update AnonymizationTask set GeneralizationIL = " + gil + ", StructureIL = " + sil + " where taskID = 'T_" + taskID + "';");
		
	    stmt.close();
	    con.close();
	}
	
	
	public static double structuralDistance(Cluster cl, int entID, Connection con, String edges_table, int noElements) throws Exception {
	    Statement stmt = con.createStatement();
	    ResultSet rs;
		int cEntID;
		int noNeighborsCEntID = 0;
		String neighborsEntID = "";
		int noNeighborsEntID = 0;
		double result = 0.0;
		Set<Integer> sNeighborsEntID = new HashSet<Integer>();
		
		stmt.execute("select Node2 from " + edges_table + " where Node1 = " + entID + ";");
		rs = stmt.getResultSet();
		while (rs.next()) {
			sNeighborsEntID.add(rs.getInt(1));
		}
		rs.close();

		Iterator i = cl.getIterator();
		int count = 0;
		while (i.hasNext()) {
			cEntID = (Integer)i.next();
			int q, t;
			
			Set<Integer> sNeighborsIntersect = new HashSet<Integer>();
			Set<Integer> sNeighborsReunion = new HashSet<Integer>(sNeighborsEntID);
						
			stmt.execute("select Node2 from " + edges_table + " where Node1 = " + cEntID + ";");
			rs = stmt.getResultSet();
			while(rs.next()) {
				int n = rs.getInt(1);
				sNeighborsReunion.add(n);
				sNeighborsIntersect.add(n);
			}
			rs.close();
			sNeighborsReunion.remove(entID);
			sNeighborsReunion.remove(cEntID);
			sNeighborsIntersect.retainAll(sNeighborsEntID);
			
			q = sNeighborsIntersect.size(); // matches (1)
			t = noElements - 2 - sNeighborsReunion.size(); // matches (0)
			
			// System.out.println("Compared " + entID + " with " + cEntID);
			// System.out.println("q = " + q + ", t = " + t);
			
			result += (q+t)/(noElements + 0.0);
			count++; 
		}
		return 1 - result/count;
	}
	
	
	public static Connection DBConnect (String dbname) {
		try {
        	Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");

            String database = "jdbc:odbc:Driver={Microsoft Access Driver (*.mdb)};DBQ=" +
            				   dbname + ";DriverID=22;READONLY=true}";
            // get the connection from the DriverManager
            Connection con = DriverManager.getConnection(database ,"",""); 
            return con;
        }
        catch (Exception e) {
        	System.out.println("Error: " + e);
        	return null;
        }
	}


	public static int selectSeed(Connection con, String tableName, int noElements, int noCurCl) throws Exception {
	// select the seed to start a new cluster
		 Statement stmt = con.createStatement();
		 String s = "select Node1, count(*) from " + edges_table + " " +
		 			"where Node1 in (select ID from " + ent_table + " where placed=false) and Node2 in (select ID from " + tableName + " where placed=false) " +
		 			"group by Node1 " + 
		 			"order by count(*) desc";

		 stmt.execute(s);
		 ResultSet rs = stmt.getResultSet(); 
		 int result = -1; // if no seed is found
	     if (rs.next()) {
	     		result = rs.getInt(1);
	     }
	     else {
	     	// no more unselected connected nodes exist => select among still unselected nodes (all are unconnected!)
     		result = getRandomUnselectedNode(con, tableName, noElements, noCurCl);
	     }
	     rs.close();
	     stmt.close();
	     return result;
	}

	
	public static int getRandomUnselectedNode(Connection con, String tableName, int noElements, int noCurCl) throws Exception {
	// returns the ID of an unselected node, if one exists, or -1 otherwise.
		int result = -1; // no unselected node exists in the table
		Statement stmt = con.createStatement();
     	stmt.execute("select ID from " + tableName + " where placed=false");
   	    
   	    int newSeedIndex = (int) (Math.random() * (noElements - noCurCl*k)) ;
   	    System.out.println("seed " + newSeedIndex);
     	
	    ResultSet rs = stmt.getResultSet();
	    int count = 0;
    	boolean flag = true;
	    while ((flag = rs.next()) && count<newSeedIndex) {
	    	count++;
	    	// System.out.println(rs.getInt(1));
	    }
	    
     	if (flag)
     		result = rs.getInt(1);
		
	    rs.close();
        stmt.close();
		return result;
	}
	
	
	public static void markSelectedNode(Connection con, String tableName, int ID) throws Exception {
		 Statement stmt = con.createStatement();
		 stmt.execute("update " + tableName + " set placed = true where ID=" + ID);
	     stmt.close();
	}
	
	
	public static void createClusterTable(Connection con, String clust_table) throws Exception {
		 // new cluster table name is cl+methodName+taskID
		 Statement stmt = con.createStatement();
	     // System.out.println("create table " + clust_table + "(cl_id integer, node_id integer);");
		 stmt.execute("create table " + clust_table + "(cl_id integer, node_id integer);");
	     stmt.close();
	}


	public static void createAnonEntTable(Connection con, String anon_ent_table) throws Exception {
	 	// might be extended to accomodate sensitive attributes too => the anonymized entities table size would be the same as the microdata size!
		String s = "create table " + anon_ent_table + "(cl_ID integer, num_vertices byte, no_internal_edges short, ";
		for (int i=0; i<d.getNoQuaAttr(); i++) {
			Attribute a = d.getQuaAttribute(i);
			s += a.getName();
			s += " text, ";
		}
		s = s.substring(0, s.length()-2);
		s += ");";

		System.out.println(s);
		Statement stmt = con.createStatement();
		stmt.execute(s);
	    stmt.close();
	}


	public static void createAnonEdgesTable(Connection con, String anon_edges_table) throws Exception {
		 Statement stmt = con.createStatement();
		 stmt.execute("create table " + anon_edges_table + "(cl1 integer, cl2 integer, weight integer);");
	     stmt.close();
	}


	public static void addEntityToClusterTable(Connection con, String clTableName, int clusterId, int entityId) throws Exception {
		 Statement stmt = con.createStatement();
		 stmt.execute("insert into " + clTableName + " values (" + clusterId + "," + entityId + ");");
		 // System.out.println("insert into " + clTableName + " values (" + clusterId + "," + entityId + ");");
	     stmt.close();
	}   

		
	public static void addClusterToAnonEntTable(Connection con, String edges_table, String anon_ent_table, int clusterId, Cluster cl) throws Exception {

		Statement stmt = con.createStatement();
		stmt.execute("select count(*) from " + edges_table + 
				 	 " where Node1 in (" + cl.toString() + ") and Node2 in (" + cl.toString() + ");");
		ResultSet rs = stmt.getResultSet();
	    rs.next(); 

		String s = "insert into " + anon_ent_table + " values (" + clusterId + ", " + cl.size() + ", " +
     	 			rs.getInt(1)/2 + ", " + cl.getSummary().toString() + ");";
		// System.out.println(s);

		stmt.execute(s);
		stmt.close();
	}
	

	public static void populateAnonEdgesTable(Connection con, String edges_table, String anon_edges_table) throws Exception {
		for (int i=0; i<(cl.length-1) && cl[i]!=null && cl[i].size()>0; i++)
			for (int j=i+1; j<cl.length && cl[j]!=null && cl[j].size()>0; j++) {
				// compute number of edges between entities in cl[i] and cl[j]
				Statement stmt = con.createStatement();
				stmt.execute("select count(*) from " + edges_table + 
					     	 " where Node1 in (" + cl[i].toString() + ") and Node2 in (" + cl[j].toString() + ");");
				ResultSet rs = stmt.getResultSet();
	    		rs.next();
	    		int weight = rs.getInt(1);
	    		if (weight > 0) {
					// System.out.println(i + " " + j + " " + weight);
					stmt.execute("insert into " + anon_edges_table + " values (" + (i+1) + ", " + (j+1) + ", " + weight + ");");
					stmt.execute("insert into " + anon_edges_table + " values (" + (j+1) + ", " + (i+1) + ", " + weight + ");");
	    		}
				stmt.close();			
			}
	}


	public static double quantifyGeneralizationInfoLoss(ClusterSummary csAll) {
		// can be optimzed (if searching the positions in hierarchies for csAll components only once!)
		double result = 0;
		for (int i=0; i<(cl.length-1) && cl[i]!=null && cl[i].size()>0; i++)
			result += cl[i].size() * cl[i].getSummary().InformationLoss(csAll);
		return result;
	}


	public static double quantifyStructureInfoLoss(Connection con, String anon_ent_table, String anon_edges_table, int noElements) throws Exception {
		double internalStrLoss = 0;
		double externalStrLoss = 0;
		
		Statement stmt = con.createStatement();
		
		for (int i=0; i<(cl.length-1) && cl[i]!=null && cl[i].size()>0; i++) {
			int no_poss_int_edges = cl[i].size() * (cl[i].size() - 1) / 2;

			stmt.execute("select no_internal_edges from " + anon_ent_table + " where cl_ID =" + (i+1) + ";");
			// System.out.println("select no_internal_edges from " + anon_ent_table + " where cl_ID =" + (i+1) + ";");
			ResultSet rs1 = stmt.getResultSet();
			rs1.next();
			int no_int_edges = rs1.getInt(1);
			
			internalStrLoss +=  (2 * no_int_edges * (1 - no_int_edges / (no_poss_int_edges + 0.0)));
			// System.out.println("Cluster " + (i+1) + " internal SIL: " + internalStrLoss);
			rs1.close();
			
			stmt.execute("select cl2, weight from " + anon_edges_table + " where cl1=" + (i+1) + ";");
			// System.out.println("select cl2, weight from " + anon_edges_table + " where cl1=" + (i+1) + ";");
			ResultSet rs2 = stmt.getResultSet();
			while (rs2.next()) {
				int no_poss_ext_edges = cl[i].size() * cl[rs2.getInt(1)-1].size();
				int no_ext_edges = rs2.getInt(2);
				externalStrLoss +=  (2 * no_ext_edges * (1 - no_ext_edges / (no_poss_ext_edges + 0.0)));
			}
			rs2.close();
			// System.out.println("Cluster " + (i+1) + " external SIL: " + externalStrLoss);
		}
		
		stmt.close();
		// modify similarly Zheleva and SNKAnonymization!!!
		return (internalStrLoss + externalStrLoss/2.0) / (noElements * (noElements - 1) / 2);
	}

} // class SNKAnonymization


/* NOTE 1: did not check, very slow computation of the exact csAll cluster summary! 	    
stmt.execute("select ID from " + ent_table);
rs = stmt.getResultSet();
if (rs.next()) {
	e = new Entity(d, con, ent_table, rs.getInt(1));
	csAll.updateSummaryFromEntity(e);
}
while (rs.next()) {
	e = new Entity(d, con, ent_table, rs.getInt(1));
	csAll = csAll.updateSummaryWithNewEntity(e);
}
rs.close();
*/
		